'use strict';
var superagent = require('superagent');
var Immutable = require('immutable');
var cuid = require('cuid');
function self() {
    return Wish;
}
var Wish = {
    look: function (wish, branches) {
        if (!(branches.Available && branches.Pending && branches.Error))
            throw new Error('Wish.look must be given all three of Available, Pending, and Error');
        return branches[wish.get('state')](wish.get('value'));
    },
    retriever: function (wish, blank) {
        return [
            self().look(wish, {
                Available: v => key => v.get(key),
                Error: e => key => blank.get(key),
                Pending: () => key => blank.get(key)
            }),
            wish.get('state') === 'Error'
        ];
    },
    Available: function (value) {
        return Immutable.Map({
            value: value,
            state: 'Available'
        });
    },
    Error: function (value) {
        return Immutable.Map({
            value: value,
            state: 'Error'
        });
    },
    Pending: function () {
        return Immutable.Map({ state: 'Pending' });
    },
    performOnce: function (world, path, action) {
        var existing = world.findExistingWish(path);
        if (existing) {
            return existing;
        } else {
            world.trackLatentPath(path);
            world.performAction(action);
            return self().Pending();
        }
    },
    seq2: function (wish, wishFactory) {
        return self().look(wish, {
            Pending: () => self().Pending(),
            Error: e => self().Error(e),
            Available: v => wishFactory(v)
        });
    },
    seq: function (wish, ...listOfWishFactories) {
        return listOfWishFactories.reduce(self().seq2, wish);
    }
};
module.exports = Wish;
