var Typeahead = React.createClass({
    propTypes: {
        defaultValue: React.PropTypes.string,
        minimumLength: React.PropTypes.number,
        jqueryNodeForItem: React.PropTypes.func.isRequired,
        dataSource: React.PropTypes.func.isRequired,
        onChange: React.PropTypes.func.isRequired,
        spins: React.PropTypes.bool
    },
    getDefaultProps: function () {
        return {
            defaultValue: '',
            minimumLength: 2,
            spins: false
        };
    },
    statics: {
        stringByBoldingMatchingPart: function (item, query) {
            item = item.split('');
            query = query.split('');
            result = [];
            nextQueryChar = query.shift();
            while (!_.isEmpty(item)) {
                nextItemChar = item.shift();
                if (nextItemChar.toLocaleLowerCase() == nextQueryChar.toLocaleLowerCase()) {
                    nextQueryChar = query.shift() || '';
                    result.push({
                        str: nextItemChar,
                        bold: true
                    });
                } else {
                    result.push({
                        str: nextItemChar,
                        bold: false
                    });
                }
            }
            var secondResult = [];
            var one, two = {
                    str: '',
                    bold: false
                };
            while (!_.isEmpty(result)) {
                one = two;
                two = result.shift();
                if (!one.bold && two.bold) {
                    secondResult.push('<strong>' + two.str);
                } else if (one.bold && !two.bold) {
                    secondResult.push('</strong>' + two.str);
                } else {
                    secondResult.push(two.str);
                }
            }
            if (two.bold) {
                secondResult.push('</strong>');
            }
            return secondResult.join('');
        }
    },
    render: function () {
        return (
            <div>
        <input ref='input'
                    onFocus={this.showTypeahead}
                    disabled={this.props.disabled ? 'disabled' : ''}
                    defaultValue={this.props.defaultValue}
                    onKeyDown={this.firstResultOnEnter} />
        </div>
        );
    },
    componentDidMount: function () {
        var input = $(this.refs.input.getDOMNode());
        input.autocomplete({
            source: this.autocompleteDataSource,
            minLength: this.props.minimumLength,
            appendTo: input.parent()
        }).on('autocompleteselect autocompletechange', _.debounce(this.handleSelection, 50)).data('ui-autocomplete')._renderItem = this.autocompleteRenderItem;
        input.autocomplete('widget').addClass('org-deduper');
        if (this.props.spins && !this.spinner) {
            this.setupSpinner();
        }
    },
    componentWillReceiveProps: function (newProps) {
        if (!this.spinner && newProps.spins) {
            this.setupSpinner();
        }
    },
    componentWillUnmount: function () {
        $(this.refs.input.getDOMNode()).autocomplete('destroy');
        if (this.spinner) {
            this.spinner.stop();
        }
    },
    showTypeahead: function () {
        var input = $(this.refs.input.getDOMNode());
        input.autocomplete('search', input.val());
    },
    setupSpinner: function () {
        var input = $(this.refs.input.getDOMNode());
        this.spinner = new Spinner({
            length: 4,
            radius: 7,
            width: 3,
            color: '#999'
        });
        input.on('autocompletesearch', this.startSpinner).on('autocompleteresponse', this.stopSpinner);
    },
    spinnerOverlay: function () {
        var input = $(this.refs.input.getDOMNode());
        var spinnerLi = input.next().find('.spinner-overlay');
        return spinnerLi[0];
    },
    forceOpenWithSpinner: function () {
        var $input = $(this.refs.input.getDOMNode());
        if ($input.next().is(':visible')) {
            return;
        }
        var widget = $input.data('uiAutocomplete');
        widget._suggest([{
                name: '',
                value: '',
                term: '',
                isExact: null,
                isSpinner: true
            }]);
    },
    startSpinner: function (event, ui) {
        this.forceOpenWithSpinner();
        var overlay = this.spinnerOverlay();
        this.spinner.spin(overlay);
        overlay.style.display = 'block';
    },
    stopSpinner: function (event, ui) {
        this.spinner.stop();
        this.spinnerOverlay().style.display = 'none';
    },
    autocompleteDataSource: function (request, response) {
        this.props.dataSource(request.term, function (results) {
            if (_.contains(_.pluck(results, 'value'), undefined)) {
                throw new Error('ABC.Typeahead\'s dataSource prop must yield objects with ' + 'a \'value\' property.' + ABC.getDeclarationErrorAddendum(this));
            }
            if (this.props.spins) {
                results.push({
                    name: '',
                    value: '',
                    term: '',
                    isExact: null,
                    isSpinner: true
                });
            }
            response(results);
        }.bind(this));
    },
    autocompleteRenderItem: function (ul, item) {
        var node = this.props.jqueryNodeForItem(item);
        if (!node.is || !node.is('li')) {
            throw new Error('ABC.Typeahead\'s jqueryNodeForItem prop must return a jQuery <li> node.' + ABC.getDeclarationErrorAddendum(this));
        }
        if (!node.has('a').length) {
            throw new Error('The jQuery node returned by ABC.Typeahead\'s jqueryNodeForItem prop ' + 'must contain an <a> somewhere in it. Because jQuery!' + ABC.getDeclarationErrorAddendum(this));
        }
        if (item.isSpinner) {
            node.addClass('spinner-overlay');
        }
        return node.appendTo(ul);
    },
    firstResultOnEnter: function (event) {
        if (event.key == 'Enter' || event.key == 'Tab') {
            event.preventDefault();
            var results = $(this.refs.input.getDOMNode()).next().children();
            if (results.length) {
                var firstResult = results.not('.spinner-overlay').first();
                this.handleSelection(null, { item: { value: firstResult.data('uiAutocompleteItem').value } });
            }
        }
    },
    handleSelection: function (event, ui) {
        var wasSelectedFromMenu = !!ui.item;
        var newValue = wasSelectedFromMenu ? ui.item.value : event.target.value;
        $(this.refs.input.getDOMNode()).val(newValue);
        this.props.onChange(newValue, wasSelectedFromMenu);
    }
});
module.exports = Typeahead;
