'use strict';
var Immutable = require('immutable');
var superagent = require('superagent');
var API = require('API');
var Router = require('Router');
var urlPrefix = '/_data/en.geo/';
function asyncMemoize(f) {
    var saved = {};
    return function (arg, options = {}, cb) {
        var memoizationKey = arg + (options.type || '');
        if (saved[memoizationKey]) {
            cb(saved[memoizationKey]);
        } else {
            f(arg, options, function (x) {
                saved[memoizationKey] = x;
                cb(x);
            });
        }
    };
}
function objectFromGeoTuple(tuple) {
    return {
        code: tuple[0],
        name: tuple[1],
        geoid: tuple[2]
    };
}
function objectsFromGeoTuples(tuples) {
    return Immutable.List(tuples.map(objectFromGeoTuple));
}
var Geo = {
    NotApplicable: -1,
    autocompleteResults: asyncMemoize(function (prefix, options, cb) {
        superagent.get(Router.urlForRoute(Router.route('geo/search', {
            lang: options.lang || 'en',
            type: options.type,
            q: prefix
        }))).end((error, response) => cb(response.body.result));
    }),
    readRegionsForCountry: (world, countryGeoid) => world.getIn([
        'Geo',
        'regionsForCountry',
        countryGeoid
    ]),
    getCountries: function (locale, cb) {
        superagent.get(Router.urlForRoute(Router.route('geo/countries', { lang: locale || 'en' }))).end((error, response) => cb(objectsFromGeoTuples(response.body.result)));
    },
    getRegionsForCountry: (world, countryGeoid) => API.get(world, [
        'Geo',
        'regionsForCountry',
        countryGeoid
    ], Router.route('geo/regions', {
        lang: world.get('Locale'),
        country_geoid: countryGeoid
    }), response => objectsFromGeoTuples(response.get('result').toJSON()))
};
module.exports = Geo;
