'use strict';
var _ = require('underscore');
var React = require('react');
var Immutable = require('immutable');
var config = require('config');
var Router = require('Router');
var Wish = require('Wish');
var Worldly = require('Worldly');
var Auth = require('dealers/Auth');
var Signin = require('dealers/Signin');
var LocationField = require('elements/LocationField');
var Mixin = {
    janrainLoginSucceeded: function () {
        this.props.onClose();
    },
    userFieldsFromJanrainData: function (userData) {
        function wanted(value, key) {
            return Immutable.Set([
                'first_name',
                'full_name',
                'email',
                'location',
                'geoid'
            ]).has(key);
        }
        return userData.filter(wanted).set('location', LocationField.blankLocation({
            location: userData.get('location'),
            geoid: userData.get('geoid')
        }));
    },
    janrainSignupInitiated: function (userData) {
        Signin.setMany(this.userFieldsFromJanrainData(userData));
        this.props.switchToJanrainFlow(true);
        this.props.goNext();
    }
};
var initialized = false;
var loaded = false;
var onLoad = () => {
};
var janrainHomeElement = null;
var Preloader = React.createClass({
    componentDidMount() {
        if (initialized) {
            return;
        }
        if (!window.janrain) {
            window.janrain = { settings: { social: {} } };
        }
        var janrain = window.janrain;
        janrain.settings.tokenUrl = Router.urlForRoute(Router.route('engage-token'), { external: true });
        janrain.settings.tokenAction = 'event';
        janrain.settings.type = 'modal';
        janrain.settings.appId = config.idealist['engage.api.id'];
        janrain.settings.appUrl = config.idealist['engage.api.app.url'];
        janrain.settings.providers = [
            'facebook',
            'googleplus'
        ];
        janrain.settings.providersPerPage = '4';
        janrain.settings.format = 'one row';
        janrain.settings.actionText = 'Sign in using your account with';
        janrain.settings.showAttribution = false;
        janrain.settings.fontColor = '#666666';
        janrain.settings.fontFamily = 'lucida grande, Helvetica, Verdana, sans-serif';
        janrain.settings.backgroundColor = '#ffffff';
        janrain.settings.width = '250';
        janrain.settings.modalBorderColor = '#000000';
        janrain.settings.modalBorderRadius = '10';
        janrain.settings.modalBorderWidth = '10';
        janrain.settings.modalBorderOpacity = '0.5';
        janrain.settings.buttonBorderColor = '#CCCCCC';
        janrain.settings.buttonBorderRadius = '5';
        janrain.settings.buttonBackgroundStyle = 'gradient';
        janrain.settings.language = '';
        janrain.settings.linkClass = 'janrainEngage';
        var e = document.createElement('script');
        e.type = 'text/javascript';
        e.id = 'janrainAuthWidget';
        var domain = config.idealist['engage.api.domain'];
        if (document.location.protocol === 'https:') {
            e.src = 'https://rpxnow.com/js/lib/' + domain + '/engage.js';
        } else {
            e.src = 'http://widget-cdn.rpxnow.com/js/lib/' + domain + '/engage.js';
        }
        var s = document.getElementsByTagName('script')[0];
        s.parentNode.insertBefore(e, s);
        janrain.ready = true;
        initialized = true;
        window.janrainWidgetOnload = () => {
            loaded = true;
            janrainHomeElement = document.getElementById('janrainEngageEmbed').parentNode;
            onLoad();
        };
    },
    render() {
        return null;
    }
});
var Authenticator = React.createClass({
    mixins: [Worldly],
    propTypes: {
        onLogin: React.PropTypes.func.isRequired,
        onSignup: React.PropTypes.func.isRequired
    },
    render() {
        return <div style={{ height: 120 }} />;
    },
    onProviderLoginToken(response) {
        Auth.logInByJanrain(this.world(), response.token);
        this.latestToken = response.token;
    },
    componentWillUpdate() {
        if (this.latestToken) {
            var attempt = Auth.loginAttemptWithJanrainToken(this.world(), this.latestToken);
            if (attempt) {
                Wish.look(attempt, {
                    Available: v => {
                        v.get('primaryKey') ? this.props.onLogin(v) : this.props.onSignup(v);
                        delete this.latestToken;
                    },
                    Pending: () => {
                    },
                    Error: () => {
                    }
                });
            }
        }
    },
    componentDidMount() {
        if (loaded) {
            this.setupEngage();
        } else {
            onLoad = this.setupEngage;
        }
    },
    componentWillUnmount() {
        if (loaded) {
            if (window.janrain && window.janrain.events) {
                [
                    'onProviderLoginToken',
                    'onProviderLoginError'
                ].forEach(event => {
                    _.each(janrain.events[event].eventHandlers, (handler, i) => {
                        if (handler === this[event]) {
                            janrain.events[event].removeHandler(i);
                        }
                    });
                });
            }
            var actualNode = document.getElementById('janrainEngageEmbed');
            if (actualNode.parentNode === this.getDOMNode()) {
                janrainHomeElement.appendChild(actualNode);
            }
        }
    },
    setupEngage() {
        var janrain = window.janrain;
        janrain.engage.signin.widget.init();
        janrain.events.onProviderLoginToken.addHandler(this.onProviderLoginToken);
        janrain.events.onProviderLoginError.addHandler(this.onProviderLoginError);
        var actualNode = document.getElementById('janrainEngageEmbed');
        if (this.isMounted()) {
            this.getDOMNode().appendChild(actualNode);
        }
    },
    onProviderLoginError() {
    }
});
module.exports = {
    Preloader,
    Authenticator,
    Mixin
};
