'use strict';
var React = require('react/addons');
var Auth = require('dealers/Auth');
var Signin = require('dealers/Signin');
var Wish = require('Wish');
var Worldly = require('Worldly');
var {I18N} = require('I18N');
var Link = require('elements/Link');
var Look = require('elements/Look');
var PaneFlow = require('elements/PaneFlow');
var Spinner = require('elements/Spinner');
var UnverifiedEmailNotice = require('elements/OrgIntake/UnverifiedEmailNotice');
var Captcha = require('./NoCaptcha');
var Engage = require('./JanrainEngage');
var SignupStartPane = require('./SignupStartPane');
var SignupNameLoc = require('./SignupNameLoc');
var SignupSecure = require('./SignupSecure');
var SignupComplete = require('./SignupComplete');
var SignupStartPane = require('./SignupStartPane');
var LoginPassword = require('./LoginPassword');
var SignupStartPane = require('./SignupStartPane');
var SignupNameLoc = require('./SignupNameLoc');
var SignupSecure = require('./SignupSecure');
var JanrainSignupComplete = require('./JanrainSignupComplete');
var flows = {
    inline: {
        janrain: [
            SignupStartPane,
            SignupNameLoc,
            JanrainSignupComplete
        ],
        signup: [
            SignupStartPane,
            SignupNameLoc,
            SignupComplete
        ]
    },
    login: [
        SignupStartPane,
        LoginPassword
    ],
    janrain: [
        SignupStartPane,
        SignupNameLoc,
        SignupSecure,
        JanrainSignupComplete
    ],
    signup: [
        SignupStartPane,
        SignupNameLoc,
        SignupSecure,
        SignupComplete
    ]
};
var UnverifiedEmail = React.createClass({
    render() {
        return (
            <div>
            <h3><I18N
                        message={'You\'re almost there!'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>You're almost there!</span>;
                        }} /></h3>
            <p><I18N
                        message={'We emailed you at {this.props.pendingVerified} with a verification link that you must click on to complete the sign up process.'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>We emailed you at {this.props.pendingVerified} with a verification link that you must click on to complete the sign up process.</span>;
                        }} />
            </p>

            <p>
              <Link endpoint="resend-verification" values={{ login: this.props.pendingVerified }}><I18N
                            message={'Resend the verification email'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Resend the verification email</span>;
                            }} /></Link>
            </p>

            <button onClick={this.props.onClose}><I18N
                        message={'Will Do'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return <span>Will Do</span>;
                        }} /></button>
        </div>
        );
    }
});
var SignupFlow = React.createClass({
    mixins: [Worldly],
    statics: { SignupStartPane },
    propTypes: { inline: React.PropTypes.bool },
    getInitialState: function () {
        return {
            useLoginFlow: false,
            useJanrainFlow: false,
            mostRecentPane: 0,
            signupFired: false,
            advancedAfterSignup: false,
            override: null
        };
    },
    switchToLoginFlow: function (bool) {
        this.setState({ useLoginFlow: bool });
    },
    switchToJanrainFlow: function (bool) {
        this.setState({ useJanrainFlow: bool });
    },
    componentWillMount: function () {
        Signin.initForm(this);
    },
    componentDidUpdate: function () {
        if (this.state.signupFired && this.signupWish()) {
            Wish.look(this.signupWish(), {
                Available: result => {
                    if (result && !this.state.advancedAfterSignup) {
                        this.setState({
                            mostRecentPane: ++this.state.mostRecentPane,
                            advancedAfterSignup: true
                        });
                    }
                },
                Pending: () => {
                },
                Error: err => {
                    this.setState({ signupFired: false });
                }
            });
        }
    },
    overrideWithUnverified() {
        this.setState({ override: this.props.inline ? UnverifiedEmailNotice : UnverifiedEmail });
    },
    signupWish() {
        if (global.location) {
            return Auth.signupAttemptWithPayload(this.world(), this._signupPayload());
        }
    },
    _wishForSignup() {
        return Auth.signUpByEmail(this.world(), this._signupPayload());
    },
    _signupPayload() {
        var fieldHash = Signin.getState();
        return fieldHash.merge(fieldHash.get('location')).set('via', window.location.href).set('cx_required', false);
    },
    signup: function () {
        this._wishForSignup();
        this.setState({ signupFired: true });
        if (this.props.inline) {
            this.world().performAction({
                name: 'SignupSecure.signup',
                transaction: ws => {
                    return ws.setIn([
                        'Auth',
                        'hideEmailVerificationMsg'
                    ], true);
                }
            });
        }
    },
    flow: function () {
        if (this.state.useLoginFlow) {
            return flows.login;
        }
        if (this.props.inline) {
            return flows.inline[this.state.useJanrainFlow ? 'janrain' : 'signup'];
        }
        if (this.state.useJanrainFlow) {
            return flows.janrain;
        }
        return flows.signup;
    },
    focus: function () {
        this.refs.flow.focus();
    },
    render: function () {
        var cls = 'signup-flow ' + (this.props.className || '');
        return (
            <div className={cls}>
            <Captcha.Preloader />
            <Engage.Preloader />
            <PaneFlow.PaneFlow {...this.props} ref="flow"
                    flow={this.flow()}
                    attemptSignup={this.signup}
                    switchToLoginFlow={this.switchToLoginFlow}
                    switchToJanrainFlow={this.switchToJanrainFlow}
                    isJanrainFlow={this.state.useJanrainFlow}
                    currentPane={this.state.mostRecentPane}
                    onChangePane={i => this.setState({ mostRecentPane: i })}
                    signupWish={this.signupWish()}
                    onUnverified={this.overrideWithUnverified}
                    override={this.state.override}
                    pendingVerified={Auth.emailAddressPendingVerification(this.world())} />
        </div>
        );
    }
});
module.exports = SignupFlow;
