'use strict';
var React = require('react');
var _ = require('underscore');
var {Map, List, fromJS} = require('immutable');
var API = require('API');
var {I18N, i18n} = require('I18N');
var Worldly = require('Worldly');
var Valid = require('abc/validators');
var Analytics = require('dealers/AnalyticsDealer');
var OrgIntakeForm = require('dealers/OrgIntakeForm');
var Form = require('elements/Forms/Form');
var OrgField = require('elements/OrgIntake/OrgField');
var NewOrgCreationForm = require('elements/OrgIntake/NewOrgCreationForm');
var OrgNameDeDuper = require('elements/OrgIntake/OrgNameDeDuper');
var Wrapper = require('elements/OrgIntake/OrgField');
var LocationField = require('elements/LocationField');
var Spinner = require('elements/Spinner');
var Org = require('schemas/Org');
var OrgTypeInput = React.createClass({
    render() {
        return (
            <div>
            <div className="styled-select">
                <select value={this.props.value} onChange={this.doChg}>
                    {_.map(Org.TYPES(), (label, intRep) => <option key={intRep} value={intRep}>{label}</option>)}
                </select>
            </div>
            <p id="are-we-eligible" className="in-form-link">
                <I18N
                        message={'<a href="/info/Help/Organizations#Eligibility">Are we eligible?</a>'}
                        context={this}
                        args={[]}
                        fallback={function () {
                            return (
                                <span>
                    <a href="/info/Help/Organizations#Eligibility">Are we eligible?</a>
                </span>
                            );
                        }} />
            </p>
        </div>
        );
    },
    doChg(evt) {
        this.props.onChange(evt.target.value);
    }
});
var OrgLocationInput = React.createClass({
    getInitialState() {
        return { value: this.props.value };
    },
    chooseLocation(location) {
        OrgIntakeForm.setField('addresses', List([location]));
        this.props.onChange([location.toJS ? location.toJS() : location]);
    },
    setValue(val) {
        this.setState({ value: [val.toJS()] });
    },
    componentWillMount() {
        this.props.element.observe((isValid, elm) => this.setState({ value: elm.value }));
    },
    render() {
        var locations = this.props.value;
        return (
            <LocationField
                element={this.props.element.members[0]}
                onSelect={this.chooseLocation}
                onChange={this.setValue}
                onBlur={evt => {
                    if (evt) {
                        if (!evt.relatedTarget) {
                            this.chooseLocation(this.state.value[0]);
                        }
                    }
                }}
                showErrors={this.props.showErrors}
                value={Map(this.state.value[0])}
                maxMenuHeight={350}
                autocompleteEndpointOptions={{ type: 'place' }} />
        );
    }
});
var ScriptPreLoader = React.createClass({
    mixins: [Worldly],
    getInitialState() {
        return { scriptsLoaded: false };
    },
    componentDidMount() {
        API.constants(this.world(), 'restrictiveRTE', 'areaOfFocus');
        var $script = require('scriptjs');
        var jquery = '//code.jquery.com/jquery-1.8.3.min.js';
        var uc = '//ucarecdn.com/widget/2.1.4/uploadcare/uploadcare.min.js';
        var wysi = '/javascript/src/vendor/forks/wysihtml5-0.4.0pre.ido.js';
        var jqui = '/javascript/min/dev/vendor/ui-1.8.23/jquery-ui-1.8.23.custom.js';
        $script([
            jquery,
            wysi
        ], 'stage1');
        $script.ready('stage1', () => {
            $script([
                jqui,
                uc
            ], 'stage2');
        });
        $script.ready('stage2', () => {
            this.setState({ scriptsLoaded: true });
        });
    },
    render() {
        return this.state.scriptsLoaded ? this.props.children : this.props.showSpinner ? <Spinner /> : null;
    }
});
var OrgIntakeCore = React.createClass({
    mixins: [Worldly],
    haveLocalStorage: typeof window !== 'undefined' && !!window.localStorage,
    childContextTypes: {
        schema: React.PropTypes.object,
        onChange: React.PropTypes.func.isRequired
    },
    getChildContext() {
        return {
            schema: OrgIntakeForm.state,
            onChange: this.handleChange
        };
    },
    getInitialState() {
        return {
            allowsEditing: true,
            showErrors: false
        };
    },
    componentDidMount() {
        try {
            var cached = this.haveLocalStorage && localStorage.getItem('OrgIntake');
            if (cached) {
                OrgIntakeForm.setMany(fromJS(JSON.parse(cached)));
            } else {
                this.updateLocalStorage();
            }
            this.forceUpdate();
        } catch (e) {
        }
    },
    handleChange(success, changedField) {
        changedField.validate();
        if (success && ([
                'type',
                'addresses'
            ].indexOf(changedField.name) !== -1 || changedField === OrgIntakeForm.state)) {
            this.updateLocalStorage();
        }
    },
    updateLocalStorage() {
        if (this.haveLocalStorage) {
            localStorage.setItem('OrgIntake', JSON.stringify({
                type: OrgIntakeForm.state.members.type.value,
                addresses: OrgIntakeForm.state.members.addresses.value
            }));
        }
    },
    render() {
        let typeValid = OrgIntakeForm.state.members.type.validate();
        let address = OrgIntakeForm.state.members.addresses.members[0];
        let addressValid = address ? address.validate() : false;
        let disableOrgName = !(typeValid && addressValid);
        let showAddButton = this.props.authed && !OrgIntakeForm.existingOrgIsSelected();
        return (
            <Form className="create-org" schema={OrgIntakeForm.state} onChange={this.handleChange}>
            <div>
                <OrgField name="type"
                        type={OrgTypeInput}
                        label={i18n('What kind of organization do you represent?')}
                        required={true} />
            </div>

            <fieldset>
                <OrgField name="addresses"
                        element={OrgIntakeForm.getMember('addresses')}
                        type={OrgLocationInput}
                        required={true}
                        showErrors={this.state.showErrors}
                        label={i18n('Where is your org located?')} />
            </fieldset>

            {!this.props.authed && (
                    <a className='unauth-upsell' href="#!" onClick={this.jumpToLogin}>
                    <I18N
                            message={'Log in or create your personal account \u203A\u203A'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Log in or create your personal account &rsaquo;&rsaquo;</span>;
                            }} />
                </a>
                )}

            {this.props.authed && (
                    <div>
                  {!OrgIntakeForm.existingOrgIsSelected() ? (
                            <OrgField name="fullName"
                                type={OrgNameDeDuper}
                                label={i18n('What\'s the name of your organization?')}
                                disabled={disableOrgName}
                                showErrors={this.state.showErrors} />
                        ) : <OrgNameDeDuper onChange={v => OrgIntakeForm.setField('fullName', v)} />}
              </div>
                )}

            <ScriptPreLoader showSpinner={this.completeFormVisible()}>
                {this.completeFormVisible() ? <NewOrgCreationForm showErrors={this.state.showErrors} /> : null}
            </ScriptPreLoader>

            {!OrgIntakeForm.state.valid && this.state.showErrors && (
                    <span className="server-error">
                <I18N
                            message={'Please check the form above for errors, then try again.'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Please check the form above for errors, then try again.</span>;
                            }} />
            </span>
                )}

            {showAddButton && (
                    <button type="button"
                        key={'addBtn' + this.completeFormVisible()}
                        onClick={this.submit}
                        {...!this.completeFormVisible() ? { className: 'disabled' } : {}}>
                    <I18N
                            message={'Add'}
                            context={this}
                            args={[]}
                            fallback={function () {
                                return <span>Add</span>;
                            }} />
            </button>
                )}
            {OrgIntakeForm.isSubmitting(this.world()) && (
                    <Spinner
                        style={{
                            marginTop: '1em',
                            marginRight: '-1em',
                            float: 'right'
                        }} />
                )}
        </Form>
        );
    },
    submit(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        if (this.completeFormVisible()) {
            var success = OrgIntakeForm.state.validate();
            this.setState({ showErrors: true });
            if (success) {
                OrgIntakeForm.setMany(fromJS(OrgIntakeForm.state.value));
                OrgIntakeForm.v2SubmitOrg(this.world());
            }
        }
    },
    jumpToLogin() {
        document.querySelector('[type=email]').focus();
    },
    completeFormVisible() {
        var org = OrgIntakeForm.selectedOrgItem();
        var visible = org && org.isAddNew && !org.isExact;
        if (visible && !this._hasTriggeredFormShownInAnalytics) {
            Analytics.trackPageview('/virtual/add-org/form-shown');
            this._hasTriggeredFormShownInAnalytics = true;
        }
        return visible;
    },
    shouldDisableOrgName() {
        return !this.locationValid() && _.isEmpty(OrgIntakeForm.getField('name'));
    }
});
module.exports = OrgIntakeCore;
