'use strict';
var React = require('react/addons');
var config = require('config');
var Auth = require('dealers/Auth');
var Router = require('Router');
var SystemMessagesDealer = require('dealers/SystemMessages');
var Worldly = require('Worldly');
var {I18N, i18n} = require('I18N');
var If = require('abc/if');
var SystemMessages = React.createClass({
    mixins: [Worldly],
    render: function () {
        var style = { display: this.isVisible() ? 'block' : 'none' };
        return (
            <div id="systemMessages" style={style}>
            <ul>
            {this.emailForVerification() && <EmailVerificationMessage email={this.emailForVerification()} onDismiss={this.dismissEmailVerification} />}
            {this.messages().toArray().map(message => (
                        <SystemMessage onDismiss={e => this.onDismiss(message)} {...(message.toJS())}>
                    {message.get('text')}
                </SystemMessage>
                    ))}
            </ul>
        </div>
        );
    },
    onDismiss: function (message) {
        this.world().performAction(SystemMessagesDealer.Dismiss(message));
    },
    dismissEmailVerification: function () {
        this.world().performAction(SystemMessagesDealer.DismissEmailVerification());
    },
    isVisible: function () {
        return this.messages().count() || this.emailVerificationMessageIsVisible();
    },
    messages: function () {
        return SystemMessagesDealer.all(this.world());
    },
    emailVerificationMessageIsVisible: function () {
        return !this.world().getIn([
            'Auth',
            'hideEmailVerificationMsg'
        ]) && this.emailForVerification();
    },
    emailForVerification: function () {
        return Auth.verificationDismissed(this.world()) && Auth.emailAddressPendingVerification(this.world());
    }
});
var EmailVerificationMessage = React.createClass({
    mixins: [Worldly],
    render: function () {
        return (
            <SystemMessage style="info" isSticky="true"
                title={i18n('You\'re almost there!')}
                onDismiss={this.props.onDismiss}>
            <I18N
                    message={'<p> We just emailed you at <em>{this.props.email}</em> with a\n            verification link that you must click on to complete the sign\n            up process.</p>\n            <p>If you cannot find this email, please check your Spam or Junk\n            folder, or <a href={this.resendUrl()}>click here to send\n            yourself another verification email</a>.</p>'}
                    context={this}
                    args={[]}
                    fallback={function () {
                        return (
                            <span><p> We just emailed you at <em>{this.props.email}</em> with a
            verification link that you must click on to complete the sign
            up process.</p>
            <p>If you cannot find this email, please check your Spam or Junk
            folder, or <a href={this.resendUrl()}>click here to send
            yourself another verification email</a>.</p></span>
                        );
                    }} />
            {config.idealist.devmode && <p><button onClick={this.terminatePendingCookie}>Devmode: Remove Pending Cookie</button></p>}
        </SystemMessage>
        );
    },
    resendUrl: function () {
        return Router.urlForRoute(Router.route('resend-verification', { login: this.props.email }));
    },
    terminatePendingCookie() {
        this.world().performAction(SystemMessagesDealer.RemoveEmailVerificationCookie());
    }
});
var SystemMessage = React.createClass({
    autoHideDelay: 5000,
    render: function () {
        var className = this.props.style + ' ' + (this.props.isSticky ? 'sticky' : '');
        var icon = '/css/default/images/systemMessages/' + this.props.style + '.png';
        return (
            <li className={className} onHover={this.setAutoDismissalTimerIfApplicable}>
            <a className="closer" onClick={this.dismiss} href="#" />
            <div className="messageBody">
                <img src={icon} alt={this.props.style} />
                <h3>{this.props.title}</h3>
                {this.props.cookieMarshalled ? <div dangerouslySetInnerHTML={{ __html: this.props.text }} /> : <div>{this.props.children}</div>}
            </div>
        </li>
        );
    },
    componentDidMount: function () {
        this.setAutoDismissalTimerIfApplicable();
    },
    setAutoDismissalTimerIfApplicable: function () {
        if (!this.props.isSticky) {
            this.clearTimeout();
            this._timeout = window.setTimeout(this.dismiss, this.autoHideDelay);
        }
    },
    dismiss: function (event) {
        event && event.preventDefault();
        this.clearTimeout();
        this.props.onDismiss();
    },
    clearTimeout: function () {
        if (this._timeout) {
            window.clearTimeout(this._timeout);
            delete this._timeout;
        }
    }
});
module.exports = SystemMessages;
