'use strict';
var Location = require('./Location');
var {i18n} = require('I18N');
import {
    Schema,
    Validation
} from 'conformist';
let {Int, Str, Bool, Enum, Map, List} = Schema;
let {Value, Length} = Validation;
const CAPITALIZED_TYPES = {
    'nonprofit': 'Nonprofit',
    'government': 'Government',
    'socialenterprise': 'SocialEnterprise',
    'recruiter': 'Recruiter',
    'consultant': 'Consultant'
};
const TYPES = function () {
    return {
        'nonprofit': i18n('Nonprofit or community organization'),
        'government': i18n('Government agency'),
        'socialenterprise': i18n('Social enterprise'),
        'recruiter': i18n('Recruitment firm'),
        'consultant': i18n('Consultant/vendor to nonprofits')
    };
};
let WebURL = function (element, context) {
    if (element.value && !/^(https?:\/\/)?([^.:/]+?\.){1,}([^.]{2,})$/.test(element.value.trim())) {
        element.addError(i18n('Please enter a valid web URL'));
        return false;
    }
    return true;
};
let Org = Map.of(Enum.of(Str).named('type').valued(Object.keys(CAPITALIZED_TYPES)).using({ default: 'nonprofit' }), List.of(Location).named('addresses').validatedBy(Length.AtLeast(1, i18n('A location is required.'))), Str.named('fullName').validatedBy(Value.Present(i18n('Please enter your organization\'s name')), Length.AtLeast(1, i18n('Please enter your organization\'s name')), Length.AtMost(255, i18n('Organization name must be fewer than 255 characters'))), Str.named('shortName').validatedBy(Value.Present(i18n('Please enter your organization\'s nickname')), Length.AtLeast(1, i18n('Please enter your organization\'s nickname')), Length.AtMost(25, i18n('Organization nickname must be fewer than 25 characters'))), Str.named('streetAddress'), Str.named('deliveryDetails'), Str.named('ein'), Str.named('website').validatedBy(WebURL), Str.named('description').using({ default: '' }).validatedBy(Value.Present(i18n('A description is required.')), Length.AtLeast(1, i18n('A description is required.'))), List.named('keywords').of(Str).validatedBy(Length.AtLeast(1, i18n('An Area of Focus is required.'))), Str.named('imageUrl'), List.named('activationActions').of(Str));
Org.TYPES = TYPES;
Org.CAPITALIZED_TYPES = CAPITALIZED_TYPES;
module.exports = Org;
